<?php

/**
 * Admin Assets.
 */
if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly.
}
if (!class_exists('WAL_Admin_Assets')) {

	/**
	 * Class.
	 */
	class WAL_Admin_Assets {

		/**
		 * Suffix.
		 * 
		 * @var string
		 */
		private static $suffix;

		/**
		 * Class Initialization.
		 */
		public static function init() {
			self::$suffix = defined('SCRIPT_DEBUG') && SCRIPT_DEBUG ? '' : '.min';

			add_action('admin_enqueue_scripts', array( __CLASS__, 'external_js_files' ), 20);
			add_action('admin_enqueue_scripts', array( __CLASS__, 'external_css_files' ));
		}

		/**
		 * Enqueue external JS files.
		 */
		public static function external_css_files() {
			$screen_ids = wal_page_screen_ids();
			$newscreenids = get_current_screen();
			$screenid = str_replace('edit-', '', $newscreenids->id);

			$screen_ids[] = 'user-edit';
			$screen_ids[] = 'profile';

			if (!in_array($screenid, $screen_ids)) {
				return;
			}

			wp_enqueue_style('wal-admin', WAL_PLUGIN_URL . '/assets/css/admin.css', array( 'wc-admin-layout' ), WAL_VERSION);
			// Lightcase.
			wp_enqueue_style('lightcase', WAL_PLUGIN_URL . '/assets/lib/lightcase/lightcase' . self::$suffix . '.css', array(), WAL_VERSION);
			// Timepicker Addon.
			wp_enqueue_style('jquery-ui-datepicker-addon', WAL_PLUGIN_URL . '/assets/lib/timepicker-addon/jquery-ui-timepicker-addon' . self::$suffix . '.css', array(), WAL_VERSION);
		}

		/**
		 * Enqueue external JS files.
		 */
		public static function external_js_files() {
			$screen_ids = wal_page_screen_ids();
			$newscreenids = get_current_screen();
			$screenid = str_replace('edit-', '', $newscreenids->id);

			$enqueue_array = array(
				'wal-admin' => array(
					'callable' => array( 'WAL_Admin_Assets', 'admin' ),
					'restrict' => in_array($screenid, $screen_ids),
				),
				'wal-refund' => array(
					'callable' => array( 'WAL_Admin_Assets', 'refund' ),
					'restrict' => 'shop_order' === $screenid,
				),
				'wal-select2' => array(
					'callable' => array( 'WAL_Admin_Assets', 'select2' ),
					'restrict' => in_array($screenid, $screen_ids),
				),
			);
			/**
			 * This hook is used to alter the admin assets.
			 * 
			 * @since 1.0
			 */
			$enqueue_array = apply_filters('wal_admin_assets', $enqueue_array);
			if (!wal_check_is_array($enqueue_array)) {
				return;
			}

			foreach ($enqueue_array as $key => $enqueue) {
				if (!wal_check_is_array($enqueue)) {
					continue;
				}

				if ($enqueue['restrict']) {
					call_user_func_array($enqueue['callable'], array());
				}
			}
		}

		/**
		 * Enqueue Admin end required JS files.
		 */
		public static function admin() {
			// Admin.   
			wp_enqueue_script('wal-admin', WAL_PLUGIN_URL . '/assets/js/admin/admin.js', array( 'jquery', 'jquery-blockui', 'wc-admin-meta-boxes' ), WAL_VERSION);
			wp_localize_script(
					'wal-admin', 'wal_admin_params', array(
				'topup_nonce' => wp_create_nonce('wal-topup-nonce'),
				'module_nonce' => wp_create_nonce('wal-module-nonce'),
				'reset_user_wallet_nonce' => wp_create_nonce('wal-reset-user-wallet-data'),
				'remove_notice_nonce' => wp_create_nonce('wal-reset-notice-nonce'),
				'topup_product_name_empty_msg' => __('Please Enter the Product Name', 'wallet-for-woocommerce'),
				'confirm_product_creation_msg' => __('Are you sure you want to create a product?', 'wallet-for-woocommerce'),
				'confirm_manual_fund_msg' => __('Are you sure you want to proceed?', 'wallet-for-woocommerce'),
				'wallet_reset_confirm_msg' => __('Are you sure you want to proceed?', 'wallet-for-woocommerce'),
				'delete_confirm_msg' => __('Are you sure you want to delete?', 'wallet-for-woocommerce'),
				'redeem_wallet_funds_nonce' => wp_create_nonce('wal-redeem-wallet-funds'),
				'gateway_funds_nonce' => wp_create_nonce('wal-gateway-funds'),
					)
			);

			// Lightcase.
			wp_enqueue_script('lightcase', WAL_PLUGIN_URL . '/assets/lib/lightcase/lightcase' . self::$suffix . '.js', array( 'jquery' ), WAL_VERSION);

			// Enhanced lightcase.
			wp_enqueue_script('wal-lightcase', WAL_PLUGIN_URL . '/assets/js/wal-lightcase-enhanced.js', array( 'jquery', 'lightcase' ), WAL_VERSION);
		}

		/**
		 * Get the wallet refund details.
		 * 
		 * @since 2.1
		 */
		public static function refund() {
			global $post;
			if (!$post->ID) {
				return;
			}

			$order = wc_get_order($post->ID);
			if (!is_object($order)) {
				return;
			}

			// Return if the order payment method is a wallet.
			if ('wal_wallet' === $order->get_payment_method()) {
				return;
			}

			// Return if the order is topup.
			if ('yes' === $order->get_meta('wal_topup_fund_credited')) {
				return;
			}

			$gateways = WC()->payment_gateways()->payment_gateways();

			// Refund.   
			wp_enqueue_script('wal-refund', WAL_PLUGIN_URL . '/assets/js/admin/refund.js', array( 'jquery', 'jquery-blockui' ), WAL_VERSION);
			wp_localize_script(
					'wal-refund', 'wal_refund_params', array(
				/* translators: %1$s- amount, %2$s - gateway name */
				'refund_button' => '<button type="button" class="button button-primary do-api-wallet-refund">' . sprintf(__('Refund %1$s via %2$s', 'woocommerce'), '<span class="wc-order-refund-amount">' . wc_price(0) . '</span>', esc_html($gateways['wal_wallet']->get_title())) . '</button>',
				'confirm_refund_msg' => __('Are you sure you want to proceed?', 'wallet-for-woocommerce'),
					)
			);
		}

		/**
		 * Enqueue select2 scripts and CSS.
		 */
		public static function select2() {

			// Timepicker Addon.
			wp_enqueue_script('jquery-ui-timpicker-addon', WAL_PLUGIN_URL . '/assets/lib/timepicker-addon/jquery-ui-timepicker-addon' . self::$suffix . '.js', array( 'jquery', 'jquery-ui-datepicker' ), WAL_VERSION);

			wp_enqueue_script('wal-enhanced', WAL_PLUGIN_URL . '/assets/js/wal-enhanced.js', array( 'jquery', 'select2' ), WAL_VERSION);
			wp_localize_script(
					'wal-enhanced', 'wal_enhanced_params', array(
				'i18n_no_matches' => __('No matches found', 'wallet-for-woocommerce'),
				'i18n_input_too_short_1' => __('Please enter 1 or more characters', 'wallet-for-woocommerce'),
				'i18n_input_too_short_n' => __('Please enter %qty% or more characters', 'wallet-for-woocommerce'),
				'i18n_input_too_long_1' => __('Please delete 1 character', 'wallet-for-woocommerce'),
				'i18n_input_too_long_n' => __('Please delete %qty% characters', 'wallet-for-woocommerce'),
				'i18n_selection_too_long_1' => __('You can only select 1 item', 'wallet-for-woocommerce'),
				'i18n_selection_too_long_n' => __('You can only select %qty% items', 'wallet-for-woocommerce'),
				'i18n_load_more' => __('Loading more results&hellip;', 'wallet-for-woocommerce'),
				'i18n_searching' => __('Searching&hellip;', 'wallet-for-woocommerce'),
				'search_nonce' => wp_create_nonce('wal-search-nonce'),
				'ajaxurl' => WAL_ADMIN_AJAX_URL,
				'calendar_image' => WC()->plugin_url() . '/assets/images/calendar.png',
					)
			);
		}
	}

	WAL_Admin_Assets::init();
}
