<?php

/**
 * Frontend Assets.
 * */
if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly.
}
if (!class_exists('WAL_Fronend_Assets')) {

	/**
	 * Class.
	 */
	class WAL_Fronend_Assets {

		/**
		 * Suffix.
		 * 
		 * @var string
		 */
		private static $suffix;

		/**
		 * In Footer.
		 * 
		 * @var bool
		 */
		public static $in_footer = false;

		/**
		 * Class Initialization.
		 */
		public static function init() {

			self::$suffix = defined('SCRIPT_DEBUG') && SCRIPT_DEBUG ? '' : '.min';

			// Enqueue script in footer.
			if ('2' == get_option('wal_advanced_frontend_enqueue_scripts_type')) {
				self::$in_footer = true;
			}

			add_action('wp_enqueue_scripts', array( __CLASS__, 'external_css_files' ), 20);
			add_action('wp_enqueue_scripts', array( __CLASS__, 'external_js_files' ), 20);
		}

		/**
		 * Enqueue external JS files.
		 */
		public static function external_js_files() {
			self::select2();

			// Frontend.
			wp_enqueue_script('wal-frontend', WAL_PLUGIN_URL . '/assets/js/frontend/frontend.js', array( 'jquery', 'jquery-blockui' ), WAL_VERSION, self::$in_footer);

			wp_localize_script(
					'wal-frontend', 'wal_frontend_params', array(
				'ajax_url' => WAL_ADMIN_AJAX_URL,
				'pagination_nonce' => wp_create_nonce('wal-pagination-nonce'),
				'partial_fund_usage_nonce' => wp_create_nonce('wal-partial-fund-usage-nonce'),
				'pagination_action_nonce' => wp_create_nonce('wal-pagination-action-nonce'),
				'currency_symbol' => get_woocommerce_currency_symbol(),
				'currency_num_decimals' => wc_get_price_decimals(),
				'currency_decimal_sep' => esc_attr(wc_get_price_decimal_separator()),
				'currency_thousand_sep' => esc_attr(wc_get_price_thousand_separator()),
				'currency_format' => esc_attr(str_replace(array( '%1$s', '%2$s' ), array( '%s', '%v' ), get_woocommerce_price_format())),
					)
			);

			// Enhanced lightcase.
			wp_enqueue_script('wal-lightcase', WAL_PLUGIN_URL . '/assets/js/wal-lightcase-enhanced.js', array( 'jquery', 'lightcase' ), WAL_VERSION);
			// Lightcase.
			wp_enqueue_script('lightcase', WAL_PLUGIN_URL . '/assets/lib/lightcase/lightcase' . self::$suffix . '.js', array( 'jquery' ), WAL_VERSION);
			//Accounting.
			wp_register_script('accounting', WC()->plugin_url() . '/assets/js/accounting/accounting.js', array( 'jquery' ), WAL_VERSION);
		}

		/**
		 * Enqueue external CSS files.
		 */
		public static function external_css_files() {
			wp_register_style('wal-inline-style', false, array(), WAL_VERSION); // phpcs:ignore
			wp_enqueue_style('wal-inline-style');

			//Add inline style.
			self::add_inline_style();

			// Lightcase.
			wp_enqueue_style('lightcase', WAL_PLUGIN_URL . '/assets/lib/lightcase/lightcase' . self::$suffix . '.css', array(), WAL_VERSION);
			// Frontend.
			wp_enqueue_style('wal-frontend', WAL_PLUGIN_URL . '/assets/css/frontend.css', array(), WAL_VERSION);

			// Module.
			wp_enqueue_style('wal-module', WAL_PLUGIN_URL . '/assets/css/module.css', array(), WAL_VERSION);
		}

		/**
		 * Add Inline style.
		 */
		public static function add_inline_style() {
			$contents = get_option('wal_advanced_custom_css', '');
			if (!$contents) {
				return;
			}

			//Add custom css as inline style.
			wp_add_inline_style('wal-inline-style', $contents);
		}

		/**
		 * Enqueue select2 scripts and CSS.
		 */
		public static function select2() {
			wp_register_script('wal-enhanced', WAL_PLUGIN_URL . '/assets/js/wal-enhanced.js', array( 'jquery', 'select2' ), WAL_VERSION);
			wp_localize_script(
					'wal-enhanced', 'wal_enhanced_params', array(
				'i18n_no_matches' => __('No matches found', 'wallet-for-woocommerce'),
				'i18n_input_too_short_1' => __('Please enter 1 or more characters', 'wallet-for-woocommerce'),
				'i18n_input_too_short_n' => __('Please enter %qty% or more characters', 'wallet-for-woocommerce'),
				'i18n_input_too_long_1' => __('Please delete 1 character', 'wallet-for-woocommerce'),
				'i18n_input_too_long_n' => __('Please delete %qty% characters', 'wallet-for-woocommerce'),
				'i18n_selection_too_long_1' => __('You can only select 1 item', 'wallet-for-woocommerce'),
				'i18n_selection_too_long_n' => __('You can only select %qty% items', 'wallet-for-woocommerce'),
				'i18n_load_more' => __('Loading more results&hellip;', 'wallet-for-woocommerce'),
				'i18n_searching' => __('Searching&hellip;', 'wallet-for-woocommerce'),
				'search_nonce' => wp_create_nonce('wal-search-nonce'),
				'ajaxurl' => WAL_ADMIN_AJAX_URL,
					)
			);
		}
	}

	WAL_Fronend_Assets::init();
}
